/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package common.timer.ejb;

import common.timer.TimerTaskCommand;
import common.timer.TimerTask;
import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.NoSuchObjectLocalException;
import javax.ejb.SessionContext;
import javax.ejb.Timer;
import javax.ejb.TimerHandle;
import javax.ejb.TimerService;

/**
 *
 * @author mauro
 */
public class TimerSvcImpl implements TimerSvc {

    private final long SECONDS = 1000L;
    private final String LOGGER_NAME = "TimerSvc";
    private SessionContext sessionCtx;

    public TimerSvcImpl(SessionContext sessionCtx) {
        this.sessionCtx = sessionCtx;
    }

// ---------------------------------------------------------------------------

    public TimerHandle scheduleTask(TimerTask task) {
        Timer t = scheduleTaskCommand(task.getFirstCommand());

        if( t != null ) {
            return t.getHandle();
        }

        return null;
    }

    public TimerHandle unscheduleTaskCommand(TimerTaskCommand taskCommand) {
        Timer t = getTaskCommandTimer(taskCommand);

        if( t != null ) {
            Logger.getLogger(LOGGER_NAME).log(Level.FINE, "Task removed : "+t.getInfo());
            t.cancel();
            return t.getHandle();
        }

        return null;
    }

    public void unscheduleTaskCommand(TimerHandle th) {
        if( th != null && th.getTimer() != null && th.getTimer().getInfo() instanceof TimerTaskCommand ) {
            Logger.getLogger(LOGGER_NAME).log(Level.FINE, "Task removed : "+th.getTimer().getInfo());
            th.getTimer().cancel();
        } else {
            Logger.getLogger(LOGGER_NAME).log(Level.FINE, "Task not removed : "+(th != null ? th.getTimer().getInfo() : "<null>") );
        }
    }

    public void onTaskTimer(Timer timer) {
        TimerTaskCommand taskCommand = (TimerTaskCommand) timer.getInfo();
        TimerTask task = null;

        Logger.getLogger(LOGGER_NAME).log(Level.INFO, "TimerTaskCommand scheduled execution : "+taskCommand.toString());
        try {
            Serializable taskData = taskCommand.getPayload();
            Class taskClass = taskCommand.getTaskExecutorClass();
            task = TimerTask.buildTask(taskClass, taskData);
            task.execute();
        } catch (Exception ex) {
            Logger.getLogger(LOGGER_NAME).log(Level.SEVERE, "Exception in TimerTask execution: "+ex, ex);
        }

        if (task != null) {
            TimerTaskCommand nextCommand = task.getNextCommand(taskCommand);
        Logger.getLogger(LOGGER_NAME).log(Level.INFO, "Task re-scheduling : "+nextCommand.toString());
            if( nextCommand != null ) {
                this.scheduleTaskCommand(nextCommand);
            }
        }
    }

// ---------------------------------------------------------------------------

    private Timer scheduleTaskCommand(TimerTaskCommand taskCommand) {
        TimerService ts = sessionCtx.getTimerService();

        Logger.getLogger(LOGGER_NAME).log(Level.FINE, "TimerTaskCommand scheduled: "+taskCommand.toString());

        return ts.createTimer(taskCommand.getSleepSecsBefore() * SECONDS, taskCommand);
    }

    private Timer getTaskCommandTimer(TimerTaskCommand taskCommand) {
        TimerService ts = sessionCtx.getTimerService();

        for( Object o: ts.getTimers() ) {
            Timer t = (Timer)o;
            try {
                TimerTaskCommand task = (TimerTaskCommand) t.getInfo();
                if ( task.equals(taskCommand) ) {
                    Logger.getLogger(LOGGER_NAME).log(Level.FINE, "Task found : "+task);
                    return t;
                }
            } catch (NoSuchObjectLocalException ex) {
                Logger.getLogger(LOGGER_NAME).log(Level.SEVERE, ex.getMessage());
            }
        }

        Logger.getLogger(LOGGER_NAME).log(Level.FINE, "Task not found, query was : "+taskCommand);
        return null;
    }

}
